/*
	Editorial by HTML5 UP
	html5up.net | @ajlkn
	Free for personal and commercial use under the CCA 3.0 license (html5up.net/license)
*/

/* Button */

input[type="submit"],
input[type="reset"],
input[type="button"],
button,
.button {
    border: solid 2px #f56a6a;
}

/* Posts */

.posts article {
    width: 40%;
}